#pragma once
#include <memory>
#include "../types/Vec2.h"
#include "Texture.h"

struct TextureAtlas
{
    std::shared_ptr<Texture> texture;
    int frameWidth = 32;
    int frameHeight = 32;

    TextureAtlas() = default;

    TextureAtlas(std::shared_ptr<Texture> tex, int texelW, int texelH)
        : texture(std::move(tex))
    {
        SetTexelSize(texelW, texelH);
    }

    void SetTexelSize(int texelWidth, int texelHeight)
    {
        frameWidth = texelWidth;
        frameHeight = texelHeight;
    }

    core::types::Vec2 GetTexelSize() const
    {
        return core::types::Vec2(frameWidth, frameHeight);
    }

    core::types::Vec2 GetFrameUV(int index) const
    {
        if (!texture || frameWidth <= 0 || frameHeight <= 0)
            return core::types::Vec2(0.0f);

        int texWidth = texture->GetSize().x;
        int texHeight = texture->GetSize().y;

        int cols = texWidth / frameWidth;
        int x = index % cols;
        int y = index / cols;

        float u = (float)(x * frameWidth) / (float)texWidth;
        float v = 1.0f - (float)((y + 1) * frameHeight) / (float)texHeight; // flip y

        return core::types::Vec2(u, v);
    }

    core::types::Vec2 GetFrameSizeUV() const
    {
        if (!texture || frameWidth <= 0 || frameHeight <= 0)
            return core::types::Vec2(0.0f);

        return core::types::Vec2(
            static_cast<float>(frameWidth) / texture->GetSize().x,
            static_cast<float>(frameHeight) / texture->GetSize().y
        );
    }

    int GetTotalFrames() const
    {
        if (!texture || frameWidth <= 0 || frameHeight <= 0)
            return 0;

        int texWidth = texture->GetSize().x;
        int texHeight = texture->GetSize().y;
        int cols = texWidth / frameWidth;
        int rows = texHeight / frameHeight;

        return cols * rows;
    }
};
